class MakaronException(Exception):
    pass


no_config_file_found_message = \
'''
No '.makaron.yml' file found.
Create one by executing 'makaron --generate-config'.
'''

class NoConfigFileFound(MakaronException):
    def __init__(self):
        message = no_config_file_found_message
        super(NoConfigFileFound, self).__init__(message)


yaml_error_config_file_parsing_message = \
'''Yaml error during config file parsing.
Please check the format of your config file.'''

class YamlErrorConfigFileParsing(MakaronException):
    def __init__(self, exc):
        message = yaml_error_config_file_parsing_message
        super(YamlErrorConfigFileParsing, self).__init__(message)


yaml_error_config_file_bad_type_message = \
'''The Yaml parsed should be a dict and it is a {}.
Please check the format of your config file.'''

class YamlErrorConfigFileBadType(MakaronException):
    def __init__(self, type_found):
        message = yaml_error_config_file_bad_type_message.format(type_found)
        super(YamlErrorConfigFileBadType, self).__init__(message)


config_bad_format_message = \
'''The config provided should be a dict and it is a {}.
Please check the format of your config.'''

class ConfigBadFormat(MakaronException):
    def __init__(self, type_found):
        message = config_bad_format_message.format(type_found)
        super(ConfigBadFormat, self).__init__(message)


rule_bad_format_message = \
'''The rule for finding version should be a string or a list of string.
Please check the format of your config.'''

class RuleBadFormat(MakaronException):
    def __init__(self):
        message = rule_bad_format_message
        super(RuleBadFormat, self).__init__(message)


versions_found_are_different_message = \
'''Versions found are different.
{}'''

class VersionsFoundAreDifferent(MakaronException):
    def __init__(self, versions):
        versions_formatted = ''
        for version in versions:
            versions_formatted += '- {}: {}\n'.format(version.file_name, version.version.get())
        message = versions_found_are_different_message.format(versions_formatted)
        super(VersionsFoundAreDifferent, self).__init__(message)


bad_version_format = \
'''The version provided is not compatible with makaron version format.
provided: {}
makaron format: [uint].[uint].[uint]'''

class BadVersionFormat(MakaronException):
    def __init__(self, version):
        message = bad_version_format.format(version)
        super(BadVersionFormat, self).__init__(message)


bad_version_component_format = \
'''The {} component of the version provided is not correct.
It should be an unsigned int.
provided: {}'''

class BadVersionComponentFormat(MakaronException):
    def __init__(self, component_type, value):
        message = bad_version_component_format.format(component_type, value)
        super(BadVersionComponentFormat, self).__init__(message)


bad_version_component_format = \
'''The rule ({}) you have provide for the file {} does not indicate the position of the version.
The string [version] should be written somewhere in the rule.'''

class MissingVersionInRule(MakaronException):
    def __init__(self, file_name, regex):
        message = bad_version_component_format.format(regex, file_name)
        super(MissingVersionInRule, self).__init__(message)


cannot_find_any_version = \
'''Cannot find any version in file {} with the pattern "{}".
Please check the pattern you have provided in your config.'''

class CannotFindAnyVersion(MakaronException):
    def __init__(self, file_name, regex):
        message = cannot_find_any_version.format(file_name, regex)
        super(CannotFindAnyVersion, self).__init__(message)



from .version import check_version_are_identical, full_version_regex_format, LocatedVersion
from .tools import *
from .exception import MissingVersionInRule, CannotFindAnyVersion

def extract_version_from_rules(rules):
    version_collected = []
    for rule in rules:
        versions = rule.collect()
        version_collected += versions
    check_version_are_identical(versions)
    return version_collected[0].version


def apply_version_to_rules(rules, new_version):
    for rule in rules:
        rule.apply(new_version)


class Rule:

    def __init__(self, file_name, version_pattern):
        if exist(r'\[version\]', version_pattern):
            version_regex = version_pattern.replace('[version]', full_version_regex_format)
        else:
            raise MissingVersionInRule(file_name, version_pattern)

        self.version_pattern = version_pattern
        self.version_regex = version_regex
        self.file_name = file_name

    def collect(self):
        content = read_file(self.file_name)
        string_versions = search_all(self.version_regex, full_version_regex_format, content)
        versions = [LocatedVersion(string_version, self.file_name) for string_version in string_versions]

        if len(versions) == 0:
            raise CannotFindAnyVersion(self.file_name, self.version_pattern)

        return versions

    def apply(self, new_version):
        content = read_file(self.file_name)
        string_new_version = new_version.get()
        content = search_all_and_replace(self.version_regex, full_version_regex_format, content, string_new_version)
        write_file(self.file_name, content)



import re

def read_file(file_name):
    try:
        stream = open(file_name, 'r')
        content = stream.read()
        stream.close()
        return content
    except IOError as e:
        raise RuntimeError('No such file: \'{0}\''.format(file_name))


def write_file(file_name, content):
    stream = open(file_name, 'w')
    stream.write(content)
    stream.close()
    return content


def exist(regex, content):
    return len(re.findall(regex, content)) > 0


def search(regex, content):
    return re.findall(regex, content)


def search_one(regex, content):
    result = re.findall(regex, content)
    if len(result) == 0:
        raise RuntimeError('Found no occurence of the following regex {} in {}'.format(regex, content))
    if len(result) > 1:
        raise RuntimeError('Found more then one occurence of the following regex {} in {}'.format(regex, content))
    return result[0]


def search_all(first_regex, second_regex, content):
    parts = search(first_regex, content)
    found = []
    for part in parts:
        found.append(search_one(second_regex, part))
    return found


def search_all_and_replace(first_regex, second_regex, content, replace_string):
    parts = search(first_regex, content)
    for part in parts:
        string_found = search_one(second_regex, part)
        new_part = part.replace(string_found, replace_string)
        content = content.replace(part, new_part)
    return content



from .config import read_config_file
from .rule import Rule
from .exception import ConfigBadFormat, RuleBadFormat

def parse_version_rules(config):

    if not isinstance(config, dict):
        raise ConfigBadFormat(type(config))

    rules = []
    for file_name, info in config.items():
        parse_version_rule(rules, file_name, info)
    return rules


def parse_version_rule(rules, file_name, info):

    if isinstance(info, str):
        rule = Rule(file_name, info)
        rules.append(rule)

    elif isinstance(info, list):
        for info_item in info:
            if isinstance(info_item, str):
                rule = Rule(file_name, info_item)
                rules.append(rule)
            else:
                raise RuleBadFormat()

    else:
        raise RuleBadFormat()



#!/usr/bin/env python

from setuptools import setup, find_packages

__version__ = '1.0.0'

setup(
    name='makaron',
    version=__version__,
    description='Version handler',
    author='Noel Martignoni',
    author_email='noel@martignoni.fr',
    url='https://gitlab.com/makaron/makaron',
    scripts=['scripts/makaron'],
    install_requires=['future', 'pyyaml'],
    packages=find_packages(exclude=['tests*']),
)



import os, yaml

from .exception import YamlErrorConfigFileParsing, NoConfigFileFound, YamlErrorConfigFileBadType

config_file_name = '.makaron.yml'

config_file_content = \
'''# {}
# more info at https://makaron.gitlab.io

setup.py: "__version__ = '[version]'"

'''.format(config_file_name)


def generate_config_file():
    with open(config_file_name, 'w') as stream:
        stream.write(config_file_content)


def read_config_file():
    if os.path.isfile(config_file_name):
        with open(config_file_name, 'r') as stream:
            try:
                content = stream.read()
                config = yaml.safe_load(content)
                if not isinstance(config, dict):
                    raise YamlErrorConfigFileBadType(type(config))

                return config
            except yaml.YAMLError as exc:
                raise YamlErrorConfigFileParsing(exc)

    else:
        raise NoConfigFileFound()



import re
from .exception import MakaronException, BadVersionFormat, BadVersionComponentFormat, VersionsFoundAreDifferent

full_version_regex_format = r'[0-9]+\.[0-9]+\.[0-9]+'


def check_version_are_identical(versions):
    different = False

    first_version = versions[0].version.get()
    for located in versions[1:]:
        if located.version.get() != first_version:
            different = True
            break

    if different:
        raise VersionsFoundAreDifferent(versions)


class Version():
    def __init__(self, string_version='0.0.0'):
        self.set_from_string(string_version)

    def __str__(self):
        return self.get()

    def copy(self, version):
        self.major = version.major
        self.minor = version.minor
        self.patch = version.patch

    def set_from_string(self, version):
        pattern = re.compile(full_version_regex_format)
        if not pattern.match(version):
            raise BadVersionFormat(version)

        version_spiltted = version.split('.')
        self.major = int(version_spiltted[0])
        self.minor = int(version_spiltted[1])
        self.patch = int(version_spiltted[2])

    def set_major(self, major):
        if not isinstance(major, int):
            raise BadVersionComponentFormat('major', major)
        if major < 0:
            raise BadVersionComponentFormat('major', major)
        self.major = major

    def set_minor(self, minor):
        if not isinstance(minor, int):
            raise BadVersionComponentFormat('minor', minor)
        if minor < 0:
            raise BadVersionComponentFormat('minor', minor)
        self.minor = minor

    def set_patch(self, patch):
        if not isinstance(patch, int):
            raise BadVersionComponentFormat('patch', patch)
        if patch < 0:
            raise BadVersionComponentFormat('patch', patch)
        self.patch = patch

    def increase_major(self):
        self.major += 1
        self.minor = 0
        self.patch = 0

    def increase_minor(self):
        self.minor += 1
        self.patch = 0

    def increase_patch(self):
        self.patch += 1

    def get(self):
        return '{}.{}.{}'.format(self.major, self.minor, self.patch)


class LocatedVersion():
    def __init__(self, version, file_name):
        self.version = Version(version)
        self.file_name = file_name
